<?php

///////////////////////////////////////////////////////////////////////////////
// $Id: functions.inc.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
///////////////////////////////////////////////////////////////////////////////
// $Log: functions.inc.php,v $
// Revision 1.1.1.1  2002/03/31 00:07:42  richy
// Initial import into CVS (release rel-1_0).
//
///////////////////////////////////////////////////////////////////////////////

/**
* General functions.
*
* @module         Functions
* @modulegroup    include
* @package        PICker
* @version        $Revision: 1.1.1.1 $
* @copyright      (c)2002 R.J. Vrijhof
* @magic          $Id: functions.inc.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
* @access         public
*/

/**
* Get current time in microseconds and convert to float.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @return   float   current time with microsecond precision
* @access   public
*/
function getmicrotime() {
  list($usec, $sec) = explode(" ", microtime());
  return((float)$usec + (float)$sec);
}

/**
* Convert month as string of 3 characters to 2-digit number (but with type string).
*
* Note: case insensitive!
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $month      month as string of 3 characters ("jan", "feb" etc.)
* @return   mixed     on success, string with month as 2-digit number;
*                     else false
* @see      num2month()
* @access   public
*/
function month2num($month) {
  switch (strtolower($month)) {
    case "jan" :
      return "01";
    case "feb" :
      return "02";
    case "mar" :
      return "03";
    case "apr" :
      return "04";
    case "may" :
      return "05";
    case "jun" :
      return "06";
    case "jul" :
      return "07";
    case "aug" :
      return "08";
    case "sep" :
      return "09";
    case "oct" :
      return "10";
    case "nov" :
      return "11";
    case "dec" :
      return "12";
    default :
      return false;
  }
}

/**
* Convert month as number to string of 3 characters.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $num        month as number (1-12)
* @return   mixed     on success, string with month ("Jan", "Feb", etc.);
*                     else false
* @see      month2num()
* @access   public
*/
function num2month($num) {
  switch ($num) {
    case 1 :
      return "Jan";
    case 2 :
      return "Feb";
    case 3 :
      return "Mar";
    case 4 :
      return "Apr";
    case 5 :
      return "May";
    case 6 :
      return "Jun";
    case 7 :
      return "Jul";
    case 8 :
      return "Aug";
    case 9 :
      return "Sep";
    case 10 :
      return "Oct";
    case 11 :
      return "Nov";
    case 12 :
      return "Dec";
    default :
      return false;
  }
}

/**
* Pad every octet of an IP address with zeroes until 3 characters long.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $in         input IP address
* @return   string    input IP address "zeroed"
* @access   public
*/
function zero_IP($in) {
  // Pattern for (s)printf to make every octet of an IP address patched with
  // zeroes upto 3 characters.
  $zeroed_pat = "%03d.%03d.%03d.%03d";
  // Retrieve the individual octets.
  $IP_parts = explode(".", $in);
  // Now patch each octet with zeroes.
  return sprintf(
    $zeroed_pat,
    $IP_parts[0],
    $IP_parts[1],
    $IP_parts[2],
    $IP_parts[3]
  );
}

/**
* Remove leading zeroes of every octet of an IP address.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $in         input IP address
* @return   string    input IP address "normalized"
* @access   public
*/
function normalize_IP($in) {
  // Pattern for (s)printf to "normalize" an IP address already changed by
  // (s)printf($zeroed_pat, ...), so to revert it back to normal, so to speak.
  $normal_pat = "%1d.%1d.%1d.%1d";
  // Retrieve the individual octets.
  $IP_parts = explode(".", $in);
  // Now "normalize" each octet.
  return sprintf(
    $normal_pat,
    $IP_parts[0],
    $IP_parts[1],
    $IP_parts[2],
    $IP_parts[3]
  );
}

/**
* Display an HTML heading.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    integer   $which      which heading you want (1 for <h1>, etc.)
* @param    string    $text       text to display with the heading
* @access   public
*/
function disp_header($which, $text) {
  echo "<h$which style=\"text-align: center\">$text</h$which>\n\n";
}

/**
* Display the sort link (in normal mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g, $sort_mode
* @access   public
*/
function sort_link() {
  global $g, $sort_mode;

  if ($sort_mode == "ip") {
    echo(
      "<p>".$g->a(
        $g->get("self")."?sort=count",
        "Sort on count of source IP (desc)"
      )."Sort on count</a></p>\n\n"
    );
  } else {
    echo(
      "<p>".$g->a(
        $g->get("self")."?sort=ip",
        "Sort on source IP (asc)"
      )."Sort on source IP</a></p>\n\n"
    );
  }
}

/**
* Display HTML table with the collected IP addresses (in normal mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g, $sort_mode
* @access   public
*/
function disp_list() {
  global $g, $sort_mode;

  $lg = db_lg_get_ips(($sort_mode == "ip") ? true : false);
  $prelude = db_prelude_get_ips(($sort_mode == "ip") ? true : false);
  $both = db_get_ips(($sort_mode == "ip") ? true : false);

  $temp = array();
  foreach ($lg as $key => $val) {
    if (! in_array($key, array_keys($both))) {
      $temp[] = array(
        "IP" => $key,
        "count" => $val
      );
    }
  }
  $lg = $temp;

  $temp = array();
  foreach ($prelude as $key => $val) {
    if (! in_array($key, array_keys($both))) {
      $temp[] = array(
        "IP" => $key,
        "count" => $val
      );
    }
  }
  $prelude = $temp;

  $temp = array();
  foreach ($both as $key => $val) {
    $temp[] = array(
      "IP" => $key,
      "lg_count" => $val[lg_count],
      "prelude_count" => $val[prelude_count]
    );
  }
  $both = $temp;

  $count = max(count($lg), count($prelude), count($both));
  echo "<table width=\"100%\" border=\"0\">\n  <tr>\n    <th>Loggrep</th>\n".
       "    <th>Found in both</th>\n    <th>Prelude</th>\n  </tr>\n";
  for ($cnt = 0; $cnt < $count; $cnt++) {
    echo "  <tr>\n";
    if (gettype($lg[$cnt]) == "array") {
      echo(
        "    <td>".$g->a(
          $g->get("self")."?ip=".$lg[$cnt]["IP"],
          "View detailed portscanner info about ".$lg[$cnt]["IP"]
        ).$lg[$cnt]["IP"]."</a> (logged <strong>".$lg[$cnt]["count"].
        "</strong> times)</td>\n"
      );
    } else {
      echo "    <td>&nbsp;</td>\n";
    }
    if (gettype($both[$cnt]) == "array") {
      echo(
        "    <td>".$g->a(
          $g->get("self")."?ip=".$both[$cnt]["IP"],
          "View detailed portscanner info about ".$both[$cnt]["IP"]
        ).$both[$cnt]["IP"]."</a> (logged <strong>".$both[$cnt]["lg_count"].
        "</strong> times, received <strong>".$both[$cnt]["prelude_count"].
        ")</td>\n"
      );
    } else {
      echo "    <td>&nbsp;</td>\n";
    }
    if (gettype($prelude[$cnt]) == "array") {
      echo(
        "    <td>".$g->a(
          $g->get("self")."?ip=".$prelude[$cnt]["IP"],
          "View detailed portscanner info about ".$prelude[$cnt]["IP"]
        ).$prelude[$cnt]["IP"]."</a> (received <strong>".
        $prelude[$cnt]["count"]."</strong> times in total)</td>\n"
      );
    } else {
      echo "    <td>&nbsp;</td>\n";
    }
    echo "  </tr>\n";
  }
  echo "</table>\n";
}

/**
* Display HTML toolbar for dig, host, whois, sending email and inserting a note for an IP address.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g, $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
* @see      disp_range_toolbar()
*/
function disp_toolbar() {
  global $g, $HTTP_GET_VARS, $HTTP_POST_VARS;

?>
<table border="0">
  <tr>
    <td>
      <form name="digForm" id="digForm" method="post" action="<?php echo $g->get("self");?>">
        <input type="hidden" name="ip" id="ip" value="<?php if ($HTTP_POST_VARS["ip"]) echo $HTTP_POST_VARS["ip"]; else echo $HTTP_GET_VARS["ip"];?>" />
        <input type="submit" name="action" id="action" value="Dig" />
      </form>
    </td>
    <td>
      <form name="hostForm" id="hostForm" method="post" action="<?php echo $g->get("self");?>">
        <input type="hidden" name="ip" id="ip" value="<?php if ($HTTP_POST_VARS["ip"]) echo $HTTP_POST_VARS["ip"]; else echo $HTTP_GET_VARS["ip"];?>" />
        <input type="submit" name="action" id="action" value="Host" />
      </form>
    </td>
    <td>
      <form name="whoisForm" id="whoisForm" method="post" action="<?php echo $g->get("self");?>">
        <input type="hidden" name="ip" id="ip" value="<?php if ($HTTP_POST_VARS["ip"]) echo $HTTP_POST_VARS["ip"]; else echo $HTTP_GET_VARS["ip"];?>" />
        <input type="submit" name="action" id="action" value="Whois" />
      </form>
    </td>
    <td>
      <form name="mailForm" id="mailForm">
        <input type="button" value="Mail to Abuse" onclick="blindMail()" />
      </form>
    </td>
    <td>
      <form name="noteForm" id="noteForm">
        <input type="button" value="Note" onclick="note()" />
      </form>
    </td>
  </tr>
</table>
<?php

}

/**
* Display HTML link back to normal mode.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g
* @access   public
* @see      link_ip()
*/
function link_list() {
  global $g;

  echo "\n".$g->a(
    $g->get("self"),
    "Go back to list of IP addresses"
  );
  echo "List</a>\n";
}

/**
* Display HTML link back to info mode.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g, $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
* @see      link_list()
*/
function link_ip() {
  global $g, $HTTP_GET_VARS, $HTTP_POST_VARS;

  if ($ip = $HTTP_POST_VARS["ip"]) {} elseif ($ip = $HTTP_GET_VARS["ip"]) {}
  echo $g->a(
    $g->get("self")."?ip=$ip",
    "Go back to IP info"
  );
  echo "IP info</a>\n";
}

/**
* Return nice string about a certain ICMP type.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    integer   $type   which ICMP type
* @return   string            ICMP type explanation
* @access   public
* @see      icmp_code()
*/
function icmp_type($type) {
  switch ($type) {
    case 0 :
      return "Echo reply ($type)";
    case 3 :
      return "Destination unreachable ($type)";
    case 4 :
      return "Source quench ($type)";
    case 5 :
      return "Redirect ($type)";
    case 8 :
      return "Echo ($type)";
    case 11 :
      return "Time exceeded ($type)";
    case 12 :
      return "Parameter problem ($type)";
    case 13 :
      return "Timestamp ($type)";
    case 14 :
      return "Timestamp reply ($type)";
    case 15 :
      return "Information request ($type)";
    case 16 :
      return "Information reply ($type)";
    default :
      return "Unknown ICMP type ($type)";
  }
}

/**
* Return nice string about a certain ICMP code belonging to a certain ICMP type.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    integer   $type   which ICMP type
* @param    integer   $code   which ICMP code
* @return   string            ICMP code explanation
* @access   public
* @see      icmp_type()
*/
function icmp_code($type, $code) {
  switch ($type) {
    case 0 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 3 :
      switch ($code) {
        case 0 :
          return "Net unreachable ($code)";
        case 1 :
          return "Host unreachable ($code)";
        case 2 :
          return "Protocol unreachable ($code)";
        case 3 :
          return "Port unreachable ($code)";
        case 4 :
          return "Fragmentation needed and DF set ($code)";
        case 5 :
          return "Source route failed ($code)";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 4 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 5 :
      switch ($code) {
        case 0 :
          return "Redirect datagrams for the network ($code)";
        case 1 :
          return "Redirect datagrams for the host ($code)";
        case 2 :
          return "Redirect datagrams for the TOS and network ($code)";
        case 3 :
          return "Redirect datagrams for the TOS and host ($code)";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 8 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 11 :
      switch ($code) {
        case 0 :
          return "Time to live exceeded in transit ($code)";
        case 1 :
          return "Fragment reassembly time exceeded ($code)";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 12 :
      switch ($code) {
        case 0 :
          return "Pointer indicates the error ($code)";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 13 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 14 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 15 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    case 16 :
      switch ($code) {
        case 0 :
          return "0";
        default :
          return "Unknown ICMP code ($code)";
      }
    default :
      return "Unknown ICMP type, so ICMP code ($code) meaningless";
  }
}

/**
* Display HTML table with the mail info of an IP address (in info mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $IP   IP address
* @return   mixed     nothing if successful, else false
* @global   $g, $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
* @see      MySQL::db_get_mail()
*/
function mail_info($IP = "") {
  global $g, $HTTP_GET_VARS, $HTTP_POST_VARS;

  $ip = ($IP ? $IP : ($HTTP_POST_VARS["ip"] ? $HTTP_POST_VARS["ip"] : $HTTP_GET_VARS["ip"]));
  $mail = db_get_mail($ip);
  if (gettype($mail) != "array") {
    return false;
  }
  echo "<table border=\"0\" width=\"100%\">\n  <tr><td colspan=\"9\" ".
       "align=\"center\" style=\"text-align: center; font-family: Verdana, ".
       "Arial, Helvetica, sans-serif; font-size: large; font-weight: bold;\" ".
       "bgcolor=\"#cccccc\">Mail sent to Abuse Team - $ip</td></tr>\n  <tr><th>year</th>".
       "<td width=\"20\">&nbsp;</td><th>month</th>".
       "<td width=\"20\">&nbsp;</td><th>day</th><td width=\"20\"&nbsp;</td>".
       "<th>time</th><td width=\"20\">&nbsp;</td><th>to</th></tr>\n";
  foreach ($mail as $val) {
    echo "  <tr>\n    <td>".$val[year]."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".num2month($val[month])."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".sprintf("%1d", $val[day])."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".$val[time]."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".htmlentities($val[to_addr])."</td>\n  </tr>\n";
  }
  echo "</table><br />\n\n";
}

/**
* Display HTML button to view the info of a range of IP addresses (in normal mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @access   public
*/
function range_button() {

?>
<form name="rangeForm" id="rangeForm">
  <input type="button" value="IP Range" onclick="range()" />
</form>
<?php

}

/**
* Compare two IP addresses and return the number of octets, starting from the left, that are the same.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $oneIP    first IP address
* @param    string    $twoIP    second IP address
* @return   mixed               if successful, number of same octets (integer), else false
* @access   public
*/
function count_same_octets($oneIP, $twoIP) {
  $parts1 = explode(".", $oneIP);
  $parts2 = explode(".", $twoIP);
  if (count($parts[1]) != count($parts[2])) {
    return false;
  }
  $return = 0;
  for ($cnt = 0; $cnt < count($parts1); $cnt++) {
    if (sprintf("%1d", $parts1[$cnt]) == sprintf("%1d", $parts2[$cnt])) {
      $return++;
    } else {
      break;
    }
  }
  return $return;
}

/**
* Return the octets that are different for an array of IP addresses.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    array     $IPs      IP addresses
* @return   array               list of the different octets
* @access   public
* @see      count_same_octets()
*/
function diff_octets($IPs) {
  // Get the lowest and highest IP addresses.
  $min = min($IPs);
  $max = max($IPs);
  // Count the number of same octets in the range between $min and $max.
  $count = count_same_octets($min, $max);
  $return = array();
  // Make an array with all different octets in the range of IP addresses.
  foreach ($IPs as $val) {
    // Split IP address into its octets.
    $parts = explode(".", $val);
    $temp = "";
    // Make a string consisting of the last octets that are different.
    for ($cnt = $count; $cnt < 4; $cnt++) {
      $temp .= $parts[$cnt].".";
    }
    // Remove the last dot.
    $temp = substr($temp, 0, -1);
    $return[] = $temp;
  }
  $temp = array();
  $pattern = "";
  // Make a pattern string for sprintf() that left-pads the different octets
  // with zeroes upto 3 characters.
  for ($cnt = $count; $cnt < 4; $cnt++) {
    $pattern .= "%03d.";
  }
  // Remove the last dot.
  $pattern = substr($pattern, 0, -1);
  // Make an array for the IP addresses with the different octets zeroed.
  foreach ($return as $val) {
    $parts = explode(".", $val);
    switch ($count) {
      case 0 :
        $temp[] = sprintf($pattern, $parts[0], $parts[1], $parts[2], $parts[3]);
        break;
      case 1 :
        $temp[] = sprintf($pattern, $parts[0], $parts[1], $parts[2]);
        break;
      case 2 :
        $temp[] = sprintf($pattern, $parts[0], $parts[1]);
        break;
      case 3 :
        $temp[] = sprintf($pattern, $parts[0]);
        break;
      case 4 :
        return false;
      default :
        return false;
    }
  }
  // Now, sort this array.
  sort($temp);

  $return = array();
  $pattern = "";
  // Make the same pattern as before, but now the opposite, so each octet is
  // normalized again.
  for ($cnt = $count; $cnt < 4; $cnt++) {
    $pattern .= "%1d.";
  }
  // Remove the last dot.
  $pattern = substr($pattern, 0, -1);
  // Make an array for the IP addresses with the different octets normalized.
  foreach ($temp as $val) {
    $parts = explode(".", $val);
    switch ($count) {
      case 0 :
        $return[] = sprintf($pattern, $parts[0], $parts[1], $parts[2], $parts[3]);
        break;
      case 1 :
        $return[] = sprintf($pattern, $parts[0], $parts[1], $parts[2]);
        break;
      case 2 :
        $return[] = sprintf($pattern, $parts[0], $parts[1]);
        break;
      case 3 :
        $return[] = sprintf($pattern, $parts[0]);
    }
  }
  return $return;
}

/**
* Display HTML toolbar for dig, host, whois and sending email (in range mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g, $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
* @see      disp_toolbar()
*/
function disp_range_toolbar() {
  global $g, $HTTP_GET_VARS, $HTTP_POST_VARS;

?>
<table border="0">
  <tr>
    <td>
      <form name="digForm" id="digForm">
        <input type="button" value="Dig" onclick="pickIP('Dig')" />
      </form>
    </td>
    <td>
      <form name="hostForm" id="hostForm">
        <input type="button" value="Host" onclick="pickIP('Host')" />
      </form>
    </td>
    <td>
      <form name="whoisForm" id="whoisForm">
        <input type="button" value="Whois" onclick="pickIP('Whois')" />
      </form>
    </td>
    <td>
      <form name="mailForm" id="mailForm">
        <input type="button" value="Mail to Abuse" onclick="blindMailR()" />
      </form>
    </td>
  </tr>
</table>

<?php

}

/**
* Display HTML form to edit the automated email to Abuse (in writeMail mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $g, $HTTP_GET_VARS
* @access   public
* @see      writeMail_nl(), writeMailR(), writeMailR_nl(), MySQL::db_lg_get_id(), MySQL::db_prelude_get_id(), Loggrep::lg_info_txt(), Prelude::prelude_info_txt()
*/
function writeMail() {
  global $config, $g, $HTTP_GET_VARS;

?>
<form name="writeMail" id="writeMail" method="post" action="<?php echo $g->get("self");?>">
  <input type="hidden" name="ip" id="ip" value="<?php echo $HTTP_GET_VARS["ip"];?>" />
  <table border="0">
    <tr>
      <td>
        From
      </td>
      <td>
        <input type="text" name="from" id="from" size="50" maxlength="60" value="<?php echo $config[from_addr];?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        To
      </td>
      <td>
        <input type="text" name="to" id="to" size="50" maxlength="60" value="<?php echo "Abuse Team &lt;".rawurldecode($HTTP_GET_VARS["address"])."&gt;";?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        Subject
      </td>
      <td>
        <input type="text" name="subject" id="subject" size="50" maxlength="80" value="<?php echo "Complaint about IP address ".rawurldecode($HTTP_GET_VARS["ip"]);?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <textarea name="body" id="body" rows="20" cols="120" wrap="virtual">L.S.,

By performing a whois query I managed to get your abuse email address.
I wish to register a complaint about the person using the IP address <?php echo $HTTP_GET_VARS["ip"];?> at (a) certain time(s).
Below you'll find more info about when, where, what and who (all date/time in Greenwich Mean Time/Universal Time Coordinated).
<?php

    if (db_lg_get_id($HTTP_GET_VARS["ip"])) {
      echo lg_info_txt();
    }
    if (db_prelude_get_id($HTTP_GET_VARS["ip"])) {
      echo prelude_info_txt();
    }

?>


I hope you can do something about this person, so he/she stops nagging my system (and me).

Thank you in advance for your trouble.

Best regards,
Richard Vrijhof</textarea>
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <input type="submit" name="action" id="action" value="Send mail" />
      </td>
    </tr>
  </table>
</form>
<?php

}

/**
* Display HTML form to edit the Dutch automated email to Abuse (in writeMail mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $g, $HTTP_GET_VARS
* @access   public
* @see      writeMail(), writeMailR(), writeMailR_nl(), MySQL::db_lg_get_id(), MySQL::db_prelude_get_id(), Loggrep::lg_info_txt(), Prelude::prelude_info_txt()
*/
function writeMail_nl() {
  global $config, $g, $HTTP_GET_VARS;

?>
<form name="writeMail" id="writeMail" method="post" action="<?php echo $g->get("self");?>">
  <input type="hidden" name="ip" id="ip" value="<?php echo $HTTP_GET_VARS["ip"];?>" />
  <table border="0">
    <tr>
      <td>
        From
      </td>
      <td>
        <input type="text" name="from" id="from" size="50" maxlength="60" value="<?php echo $config[from_addr];?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        To
      </td>
      <td>
        <input type="text" name="to" id="to" size="50" maxlength="60" value="<?php echo "Abuse Team &lt;".rawurldecode($HTTP_GET_VARS["address"])."&gt;";?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        Subject
      </td>
      <td>
        <input type="text" name="subject" id="subject" size="50" maxlength="80" value="<?php echo "Klacht over IP-adres ".rawurldecode($HTTP_GET_VARS["ip"]);?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <textarea name="body" id="body" rows="20" cols="120" wrap="virtual">L.S.,

Door een whois-query uit te voeren ben ik achter dit abuse-mailadres gekomen.
Ik wil graag een klacht indienen betreffende iemand die het IP-adres <?php echo rawurldecode($HTTP_GET_VARS["ip"]);?> heeft gebruikt op (een) zeker(e) moment(en).
Onderstaand vindt u meer info over wanneer, waar, wat en wie (alle data in Greenwich Mean Time/Universal Time Coordinated).
<?php

    if (db_lg_get_id($HTTP_GET_VARS["ip"])) {
      echo lg_info_txt();
    }
    if (db_prelude_get_id($HTTP_GET_VARS["ip"])) {
      echo prelude_info_txt();
    }

?>


Ik hoop dat u iets kunt ondernemen tegen deze persoon, zodat hij/zij stopt mijn PC (en mij) lastig te vallen.

Alvast bedankt voor de moeite.

Met vriendelijke groet,
Richard Vrijhof</textarea>
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <input type="submit" name="action" id="action" value="Send mail" />
      </td>
    </tr>
  </table>
</form>
<?php

}

/**
* Display HTML form to edit the automated email to Abuse (in writeMailR mode).
*
* Note: this is meant especially if you see a range of IP addresses and you
*       suspect they have been used by one and the same person. Very handy!
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $g, $HTTP_GET_VARS
* @access   public
* @see      writeMail(), writeMail_nl(), writeMailR_nl(), MySQL::db_lg_get_id(), MySQL::db_prelude_get_id(), Loggrep::lg_info_txt(), Prelude::prelude_info_txt()
*/
function writeMailR() {
  global $config, $g, $HTTP_GET_VARS;

  $range = explode("-", $HTTP_GET_VARS["range"]);
  $IPs = array_keys(array_merge(lg_range_ips(), prelude_range_ips()));
  $temp = array();
  foreach ($IPs as $val) {
    $temp[] = zero_IP($val);
  }
  sort($temp);
  $IPs = array();
  foreach ($temp as $val) {
    $IPs[] = normalize_IP($val);
  }

?>
<form name="writeMail" id="writeMail" method="post" action="<?php echo $g->get("self");?>">
  <input type="hidden" name="range" id="range" value="<?php echo $HTTP_GET_VARS["range"];?>" />
  <table border="0">
    <tr>
      <td>
        From
      </td>
      <td>
        <input type="text" name="from" id="from" size="50" maxlength="60" value="<?php echo $config[from_addr];?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        To
      </td>
      <td>
        <input type="text" name="to" id="to" size="50" maxlength="60" value="<?php echo "Abuse Team &lt;".rawurldecode($HTTP_GET_VARS["address"])."&gt;";?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        Subject
      </td>
      <td>
        <input type="text" name="subject" id="subject" size="50" maxlength="80" value="<?php echo "Complaint about someone";?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <textarea name="body" id="body" rows="20" cols="120" wrap="virtual">L.S.,

By performing a whois query I managed to get your abuse email address.
I wish to register a complaint about the person using several IP addresses in a certain range at (a) certain time(s).
I suspect this person to be one and the same person, but using different IP addresses, either unwantingly because he/she has a dialup connection and has decided to perform the portscan/intrusion attempt on several occasions (perhaps to reduce telephone costs), or wantingly to avoid detection. You probably have the means to check whether these IP addresses really have been used by the same person, so I'll leave that in your capable hands.
Below you'll find more info about when, where, what and who (all date/time in Greenwich Mean Time/Universal Time Coordinated).
<?php

    foreach ($IPs as $val) {
      if (db_lg_get_id($val)) {
        echo lg_info_txt($val);
      }
      if (db_prelude_get_id($val)) {
        echo prelude_info_txt($val);
      }
    }

?>


I hope you can do something about this person, so he/she stops nagging my system (and me). If these IP addresses haven't been used by the same person or the IP addresses that have been used by the same person are logged only a few times, I'm sorry to have bothered you.

Thank you in advance for your trouble.

Best regards,
Richard Vrijhof</textarea>
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <input type="submit" name="action" id="action" value="Send mail" />
      </td>
    </tr>
  </table>
</form>
<?php

}

/**
* Display HTML form to edit the Dutch automated email to Abuse (in writeMailR mode).
*
* Note: this is meant especially if you see a range of IP address and you
*       suspect they have been used by one and the same person. Very handy!
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $g, $HTTP_GET_VARS
* @access   public
* @see      writeMail(), writeMail_nl(), writeMailR(), MySQL::db_lg_get_id(), MySQL::db_prelude_get_id(), Loggrep::lg_info_txt(), Prelude::prelude_info_txt()
*/
function writeMailR_nl() {

  $range = explode("-", $HTTP_GET_VARS["range"]);
  $IPs = array_keys(array_merge(lg_range_ips(), prelude_range_ips()));
  $temp = array();
  foreach ($IPs as $val) {
    $temp[] = zero_IP($val);
  }
  sort($temp);
  $IPs = array();
  foreach ($temp as $val) {
    $IPs[] = normalize_IP($val);
  }

?>
<form name="writeMail" id="writeMail" method="post" action="<?php echo $g->get("self");?>">
  <input type="hidden" name="ip" id="ip" value="<?php echo $HTTP_GET_VARS["ip"];?>" />
  <table border="0">
    <tr>
      <td>
        From
      </td>
      <td>
        <input type="text" name="from" id="from" size="50" maxlength="60" value="<?php echo $config[from_addr];?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        To
      </td>
      <td>
        <input type="text" name="to" id="to" size="50" maxlength="60" value="<?php echo "Abuse Team &lt;".rawurldecode($HTTP_GET_VARS["address"])."&gt;";?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td>
        Subject
      </td>
      <td>
        <input type="text" name="subject" id="subject" size="50" maxlength="80" value="<?php echo "Klacht over persoon";?>" />
      </td>
      <td>
        &nbsp;
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <textarea name="body" id="body" rows="20" cols="120" wrap="virtual">L.S.,

Door een whois-query uit te voeren ben ik achter dit abuse-mailadres gekomen.
Ik wil graag een klacht indienen betreffende iemand die verschillende IP-adressen in een bepaald IP-bereik heeft gebruikt op (een) zeker(e) moment(en).
Ik vermoed dat deze persoon dezelfde is onder verschillende IP-adressen, of onbewust omdat hij/zij een dialup-verbinding heeft en de portscans/intrusion-pogingen wilde "uitsmeren" over meerdere dagen (misschien om de telefoonkosten te drukken), of bewust om detectie van mijn kant te voorkomen. Ik neem aan dat u de mogelijkheid heeft om te checken of deze IP-adressen daadwerkelijk door dezelfde persoon gebruikt zijn, dus dat vertrouw ik met een gerust hart aan u toe.
Onderstaand vindt u meer info over wanneer, waar, wat en wie (alle data in Greenwich Mean Time/Universal Time Coordinated).
<?php

    foreach ($IPs as $val) {
      if (db_lg_get_id($val)) {
        echo lg_info_txt($val);
      }
      if (db_prelude_get_id($val)) {
        echo prelude_info_txt($val);
      }
    }

?>


Ik hoop dat u iets kunt ondernemen tegen deze persoon, zodat hij/zij stopt mijn PC (en mij) lastig te vallen. Als deze IP-adressen niet zijn gebruikt door dezelfde persoon of de IP-adressen die wel zijn gebruikt door dezelfde persoon zijn maar een paar keer gelogt, dan spijt het mij u lastig gevallen te hebben.

Alvast bedankt voor de moeite.

Met vriendelijke groet,
Richard Vrijhof</textarea>
      </td>
    </tr>
    <tr>
      <td colspan="3">
        <input type="submit" name="action" id="action" value="Send mail" />
      </td>
    </tr>
  </table>
</form>
<?php

}

/**
* Display HTML form to pick an IP address from an option list (in pickIP mode).
*
* Note: dig, host, whois etc. can only be performed on one IP address, not a
*       range. That's why you have to pick an IP address first that's existent
*       within this range.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $HTTP_GET_VARS
* @access   public
* @see      count_same_octets(), Loggrep::lg_range_ips(), Prelude::prelude_range_ips()
*/
function pickIP() {
  global $HTTP_GET_VARS;

?>
<p>Pick the IP address you want to check from the list:</p>

<form name="pickIP" id="pickIP">
  <?php

  $range = explode("-", $HTTP_GET_VARS["range"]);
  $count = count_same_octets($range[0], $range[1]);
  $parts = explode(".", $range[0]);
  $first = "";
  for ($cnt = 0; $cnt < $count; $cnt++) {
    $first .= $parts[$cnt].".";
  }
  echo $first;

?><select name="last" id="last" size="1" onchange="processIP('<?php echo $first;?>')">
<?php

  $IPs = array_keys(array_merge(lg_range_ips(), prelude_range_ips()));
  $last = diff_octets($IPs);
  foreach ($last as $val) {
    echo "    <option>$val</option>\n";
  }

?>
  </select>
  <input type="button" value="OK" onclick="processIP('<?php echo $first;?>')" />
  <input type="button" value="Cancel" onclick="window.close()" />
</form>
<?php

}

/**
* Display HTML table with the notes about an IP address (in info mode).
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $g, $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
* @see      MySQL::db_get_notes()
*/
function disp_notes($IP = "") {
  global $g, $HTTP_GET_VARS, $HTTP_POST_VARS;

  $ip = ($IP ? $IP : ($HTTP_POST_VARS["ip"] ? $HTTP_POST_VARS["ip"] : $HTTP_GET_VARS["ip"]));
  $notes = db_get_notes($ip);
  if (gettype($notes) != "array") {
    return false;
  }
  echo "<table border=\"0\" width=\"100%\">\n  <tr><td colspan=\"9\" ".
       "align=\"center\" style=\"text-align: center; font-family: Verdana, ".
       "Arial, Helvetica, sans-serif; font-size: large; font-weight: bold;\" ".
       "bgcolor=\"#cccccc\">Notes - $ip</td></tr>\n  <tr><th>year</th>".
       "<td width=\"20\">&nbsp;</td><th>month</th>".
       "<td width=\"20\">&nbsp;</td><th>day</th><td width=\"20\"&nbsp;</td>".
       "<th>time</th><td width=\"20\">&nbsp;</td><th>note</th></tr>\n";
  foreach ($notes as $val) {
    echo "  <tr>\n    <td>".$val[year]."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".num2month($val[month])."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".sprintf("%1d", $val[day])."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".$val[time]."</td>\n    <td>&nbsp;</td>\n".
         "    <td>".htmlentities($val[note])."</td>\n  </tr>\n";
  }
  echo "</table><br />\n\n";
}

/**
* Display some HTML with the copyright notice.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $g
* @access   public
*/
function copyright() {
  global $config, $g;

?>

<hr size="1" width="250" align="center" />
<div align="center" style="font-size : small">
  PICker v<?php echo $config[version];?> is &copy; 2002 <?php echo $g->a("http://www.rjvrijhof.com/", "Go to my site");?>R.J. Vrijhof</a>
</div>
<?php

}

?>
