<?php

///////////////////////////////////////////////////////////////////////////////
// $Id: loggrep.inc.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
///////////////////////////////////////////////////////////////////////////////
// $Log: loggrep.inc.php,v $
// Revision 1.1.1.1  2002/03/31 00:07:42  richy
// Initial import into CVS (release rel-1_0).
//
///////////////////////////////////////////////////////////////////////////////

/**
* Handle everything loggrep-related.
*
* @module         Loggrep
* @modulegroup    include
* @package        PICker
* @version        $Revision: 1.1.1.1 $
* @copyright      (c)2002 R.J. Vrijhof
* @magic          $Id: loggrep.inc.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
* @access         public
*/

/**
* Return HTML table with Loggrep logs of a source IP.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $IP   source IP address
* @return   mixed     on success HTML table, else false
* @global   $config, $HTTP_GET_VARS
* @access   public
* @see      MySQL::db_lg_get_log()
*/
function lg_info($IP = "") {
  global $config, $HTTP_GET_VARS;

  $return = "";
  $ip = ($IP ? $IP : $HTTP_GET_VARS["ip"]);
  $lg_collect = db_lg_get_log($ip);
  $return .= "\n<br />\n\n<table border=\"0\" width=\"100%\">\n  <tr>".
    "<td colspan=\"13\" align=\"center\" style=\"text-align: center; ".
    "font-family: Verdana, Arial, Helvetica, sans-serif; font-size: large; ".
    "font-weight: bold;\" bgcolor=\"#cccccc\">Loggrep - $ip</td></tr>\n  <tr>".
    "<th>year</th><td width=\"20\">&nbsp;</td><th>month</th>".
    "<td width=\"20\">&nbsp;</td><th>day</th><td width=\"20\">&nbsp;</td>".
    "<th>time</th><td width=\"20\">&nbsp;</td><th>DestIP</th>".
    "<td width=\"20\">&nbsp;</td><th>SrcPort</th><td width=\"20\">&nbsp;</td>".
    "<th>DestPort</th></tr>\n";
  foreach ($lg_collect as $val) {
    $return .= "  <tr>".
         "<td>".$val[year]."</td><td>&nbsp;</td>".
         "<td>".num2month($val[month])."</td><td>&nbsp;</td>".
         "<td>".sprintf("%1d", $val[day])."</td><td>&nbsp;</td>".
         "<td>".$val[time]."</td><td>&nbsp;</td>".
         "<td>".$config[own_IP]."</td><td>&nbsp;</td>".
         "<td>".$val[srcport]."</td><td>&nbsp;</td>".
         "<td>".$val[destport]."</td>".
         "</tr>\n";
  }
  $return .= "</table>\n";
  return $return;
}

/**
* Return Loggrep logs of a source IP as text (for automated email).
*
* Note: the time will be converted to GMT.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $IP   source IP address
* @return   mixed     on success Loggrep logs, else false
* @global   $config, $HTTP_GET_VARS
* @access   public
* @see      MySQL::db_lg_get_log()
*/
function lg_info_txt($IP = "") {
  global $config, $HTTP_GET_VARS;

  $ip = ($IP ? $IP : $HTTP_GET_VARS["ip"]);
  $lg_collect = db_lg_get_log($ip);
  $return = "\n****** $ip - Firewall Log ******\n";
  foreach ($lg_collect as $val) {
    $return .= "Logged: ".gmdate(
      "M j, Y H:i:s",
      mktime(
        substr($val[time], 0, 2),
        substr($val[time], 3, 2),
        substr($val[time], 6, 2),
        $val[month],
        $val[day],
        $val[year]
      )
    )." GMT - src IP $ip, dest IP ".$config[own_IP].", TCP src port ".
    $val[srcport].", TCP dest port ".$val[destport]."\n";
  }
  return $return;
}

/**
* Return array of found source IPs in Loggrep db table for a range of IPs.
*
* Note: works only in range mode.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @return   array     IP address as key, received count as value
* @global   $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
* @see      Functions::zero_IP(), MySQL::db_lg_get_ips()
*/
function lg_range_ips() {
  global $HTTP_GET_VARS, $HTTP_POST_VARS;

  $return = array();
  $range = explode("-", ($HTTP_POST_VARS["range"] ? $HTTP_POST_VARS["range"] : $HTTP_GET_VARS["range"]));
  $begin = zero_IP($range[0]);
  $end = zero_IP($range[1]);
  $ips = db_lg_get_ips(true);
  foreach ($ips as $key => $val) {
    if (zero_IP($key) >= $begin && zero_IP($key) <= $end) {
      $return[$key] = $val;
    }
  }
  return $return;
}

/**
* Check for new Loggrep files and update PICker database.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $sort_mode
* @access   public
* @see      MySQL::db_lg_get_last_access(), MySQL::db_lg_set_last_access(), MySQL::db_lg_get_ips(), Functions::month2num(), MySQL::db_lg_inc_count(), MySQL::db_lg_new_ip(), MySQL::db_lg_new_log()
*/
function loggrep() {
  global $config, $sort_mode;

  // Regexp to extract the fields from a line of a loggrep file. Should work
  // without changing. If you do, it could be the constant definitions
  // immediately following have to be changed also.
  $pattern = "<tr><td>([^<]+)</td><td>([^<]+)</td><td>([^<]+)</td>".
             "(<td>[^<]*</td>){3}<td>([^<]+)</td><td>([^<]+)</td>".
             "<td>[^<]+</td><td>([^<]+)</td></tr>";

  // Field of the regexp $pattern (between parenthizes) to retrieve the
  // month.
  define(LG_MONTH, 1);

  // Field of the regexp $pattern (between parenthizes) to retrieve the day.
  define(LG_DAY, 2);

  // Field of the regexp $pattern (between parenthizes) to retrieve the time.
  define(LG_TIME, 3);

  // Field of the regexp $pattern (between parenthizes) to retrieve the
  // source IP.
  define(LG_SRCIP, 5);

  // Field of the regexp $pattern (between parenthizes) to retrieve the
  // source port.
  define(LG_SRCPORT, 6);

  // Field of the regexp $pattern (between parenthizes) to retrieve the
  // destination port.
  define(LG_DESTPORT, 7);

  db_open();
  $last = db_lg_get_last_access();
  if (! $last) {
    $last = "0000-00-00";
  }

  $lg_SRCIPs = db_lg_get_ips(($sort_mode == "ip") ? true : false);

  // Get a list of all Loggrep files.
  $dp = opendir($config[lg_dir]);
  while ($dir = readdir($dp)) {
    // Only the Loggrep files.
    if (eregi("(.+)\.gz$", $dir, $regs)) {
      // Only the new files.
      if (
        substr($last, 0, 4).substr($last, 5, 2).substr($last, 8, 2) < $regs[1]
      ) {
        db_lg_set_last_access(substr($regs[1], 0, 4)."-".substr($regs[1], 4, 2)."-".substr($regs[1], 6, 2));
        // Get the file content (gzipped, so get it with gzfile() instead of
        // file()).
        $content = gzfile($config[lg_dir]."/$dir");
        // Skip the first and last two lines from the Loggrep file.
        for ($line = 2; $line < count($content) - 2; $line++) {
          // Retrieve the fields from the line.
          if (eregi($pattern, $content[$line], $regs)) {
            // Put the source IP in $lg_SRCIPs if not already there (and in the
            // db table).
            if (! $lg_SRCIPs[$regs[LG_SRCIP]]) {
              $lg_SRCIPs[$regs[LG_SRCIP]] = 1;
              db_lg_new_ip($regs[LG_SRCIP]);
            }
            // Put the log entry into db table for Loggrep.
            db_lg_new_log(
              $regs[LG_SRCIP],
              date("Y")."-".month2num($regs[LG_MONTH])."-".$regs[LG_DAY]." ".
              $regs[LG_TIME],
              $regs[LG_SRCPORT],
              $regs[LG_DESTPORT]
            );
            // Increment the count for this IP in the db table.
            db_lg_inc_count($regs[LG_SRCIP]);
          }
        }
      }
    }
  }
}

?>
