<?php

///////////////////////////////////////////////////////////////////////////////
// $Id: prelude.inc.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
///////////////////////////////////////////////////////////////////////////////
// $Log: prelude.inc.php,v $
// Revision 1.1.1.1  2002/03/31 00:07:42  richy
// Initial import into CVS (release rel-1_0).
//
///////////////////////////////////////////////////////////////////////////////

/**
* Handle everything Prelude-related.
*
* @module         Prelude
* @modulegroup    include
* @package        PICker
* @version        $Revision: 1.1.1.1 $
* @copyright      (c)2002 R.J. Vrijhof
* @magic          $Id: prelude.inc.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
* @access         public
*/

/**
* Return HTML table with Prelude logs of a source IP.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $IP   source IP address
* @return   mixed     on success HTML table, else false
* @global   $HTTP_GET_VARS
* @access   public
* @see      MySQL::db_prelude_get_log()
*/
function prelude_info($IP = "") {
  global $HTTP_GET_VARS;

  $return = "";
  $ip = ($IP ? $IP : $HTTP_GET_VARS["ip"]);
  if (! $prelude = db_prelude_get_log($ip)) {
    echo "Could not retrieve event from Prelude log table<br />\n";
    return false;
  }
  $return .= "\n<br />\n\n<table border=\"0\" width=\"100%\">\n  <tr>".
       "<td colspan=\"19\" align=\"center\" ".
       "style=\"font-family: Verdana, Arial, Helvetica, ".
       "sans-serif; font-size: large; font-weight: bold;\" ".
       "bgcolor=\"#cccccc\">Prelude - $ip</td></tr>".
       "\n  <tr><th>year</th>".
       "<td width=\"20\">&nbsp;</td><th>month</th>".
       "<td width=\"20\">&nbsp;</td><th>day</th><td width=\"20\">&nbsp;</td>".
       "<th>time</th><td width=\"20\">&nbsp;</td><th>DestIP</th>".
       "<td width=\"20\">&nbsp;</td><th>TCPSrc/ICMPType</th>".
       "<td width=\"20\">&nbsp;</td><th>TCPDest/ICMPCode</th>".
       "<td width=\"20\">&nbsp;</td><th>Plugin</th>".
       "<td width=\"20\">&nbsp;</td><th>Received</th>".
       "<td width=\"20\">&nbsp;</td><th>Message</th></tr>\n";
  foreach ($prelude as $val) {
    $return .= "  <tr>".
         "<td>".$val[year]."</td><td>&nbsp;</td>".
         "<td>".num2month($val[month])."</td><td>&nbsp;</td>".
         "<td>".sprintf("%1d", $val[day])."</td><td>&nbsp;</td>".
         "<td>".$val[time]."</td><td>&nbsp;</td>".
         "<td>".$val[destip]."</td><td>&nbsp;</td>".
         ($val[tcpsrc] && $val[tcpdest] ?
           "<td>".$val[tcpsrc]."</td><td>&nbsp;</td>".
           "<td>".$val[tcpdest]."</td><td>&nbsp;</td>" :
           "<td>".icmp_type($val[icmptype])."</td><td>&nbsp;</td>".
           "<td>".icmp_code($val[icmptype], $val[icmpcode])."</td>".
           "<td>&nbsp;</td>"
         ).
         "<td>".$val[plugin]."</td><td>&nbsp;</td>".
         "<td>".$val[received]."</td><td>&nbsp;</td>".
         "<td>".$val[message]."</td>".
         "</tr>\n";
  }
  $return .= "</table>\n";
  return $return;
}

/**
* Return Prelude logs of a source IP as text (for automated email).
*
* Note: returns time in GMT format.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @param    string    $IP   source IP address
* @return   mixed     on success Prelude logs, else false
* @global   $HTTP_GET_VARS
* @access   public
* @see      MySQL::db_prelude_get_log()
*/
function prelude_info_txt($IP = "") {
  global $HTTP_GET_VARS;

  $ip = ($IP ? $IP : $HTTP_GET_VARS["ip"]);
  if (! $prelude = db_prelude_get_log($ip)) {
    die("Could not retrieve event from Prelude log table<br />\n");
  }
  $return = "\n".str_repeat("*", 10)." $ip - Prelude Log (intrusion detection ".
            "software) ".str_repeat("*", 10)."\n";
  foreach ($prelude as $val) {
    $return .= "Logged: ".gmdate(
        "M j, Y H:i:s",
        mktime(
          substr($val[time], 0, 2),
          substr($val[time], 3, 2),
          substr($val[time], 6, 2),
          $val[month],
          $val[day],
          $val[year]
        )
      )." GMT - src IP $ip, dest IP ".$val[destip].
      ($val[tcpsrc] && $val[tcpdest] ?
        ", TCP src port ".$val[tcpsrc].", TCP dest port ".$val[tcpdest] :
        ", ICMP type \"".icmp_type($val[icmptype])."\", ICMP code \"".
        icmp_code($val[icmptype], $val[icmpcode])."\""
      ).
      ", plugin ".$val[plugin].
      ", received ".$val[received].
      (($val[received] == 1) ? " time" : " times").
      ", message \"".$val[message]."\"\n";
  }
  return $return;
}

/**
* Return array of found source IPs in Prelude db table for a range of IPs.
*
* Note: works only in range mode.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @return   array     IP address as key, received count as value
* @global   $HTTP_GET_VARS, $HTTP_POST_VARS
* @access   public
*/
function prelude_range_ips() {
  global $HTTP_GET_VARS, $HTTP_POST_VARS;

  $return = array();
  $range = explode(
    "-",
    (
      $HTTP_POST_VARS["range"] ?
      $HTTP_POST_VARS["range"] :
      $HTTP_GET_VARS["range"]
    )
  );
  $begin = zero_IP($range[0]);
  $end = zero_IP($range[1]);
  $ips = db_prelude_get_ips();
  foreach ($ips as $key => $val) {
    if (zero_IP($key) >= $begin && zero_IP($key) <= $end) {
      $return[$key] = $val;
    }
  }
  return $return;
}

/**
* Check Prelude log on new events and update PICker database.
*
* @author   Richard Vrijhof <R.J.Vrijhof@bigfoot.com>
* @global   $config, $sort_mode
* @access   public
* @see      MySQL::db_prelude_get_last_access(), MySQL::db_prelude_set_last_access(), MySQL::db_prelude_get_last_insert(), Functions::month2num(), MySQL::db_prelude_get_ips(), MySQL::db_prelude_inc_count(), MySQL::db_prelude_new_ip(), MySQL::db_prelude_new_log(), MySQL::db_prelude_get_last_insert()
*/
function prelude() {
  global $config, $sort_mode;

  // Field of the regexp pattern (between parenthizes) to retrieve the year.
  define(PRELUDE_YEAR, 5);

  // Field of the regexp pattern (between parenthizes) to retrieve the month.
  define(PRELUDE_MONTH, 2);

  // Field of the regexp pattern (between parenthizes) to retrieve the day.
  define(PRELUDE_DAY, 3);

  // Field of the regexp pattern (between parenthizes) to retrieve the time.
  define(PRELUDE_TIME, 4);

  // Field of the regexp pattern (between parenthizes) to retrieve the source
  // IP.
  define(PRELUDE_SRCIP, 1);

  // Field of the regexp pattern (between parenthizes) to retrieve the source
  // IP.
  define(PRELUDE_DESTIP, 2);

  // Field of the regexp pattern (between parenthizes) to retrieve the TCP
  // source port.
  define(PRELUDE_TCPSRC, 1);

  // Field of the regexp pattern (between parenthizes) to retrieve the TCP
  // destination port.
  define(PRELUDE_TCPDEST, 2);

  // Field of the regexp pattern (between parenthizes) to retrieve the ICMP
  // type.
  define(PRELUDE_ICMPTYPE, 1);

  // Field of the regexp pattern (between parenthizes) to retrieve the ICMP
  // code.
  define(PRELUDE_ICMPCODE, 2);

  db_open();
  $last = db_prelude_get_last_access();
  if (! $last) {
    $last = "0000-00-00 00:00:00";
  }
  $last_insert = db_prelude_get_last_insert();
  if (! $last_insert) {
    $last_insert = "0000-00-00 00:00:00";
  }

  // Get all logged Prelude source IPs as an array with the IP as key, received
  // as value.
  $prelude_SRCIPs = db_prelude_get_ips(($sort_mode == "ip") ? true : false);

  // If the file modified time is newer, there must be new entries.
  if (strcmp(
    $last,
    date(
      "Y-m-d H:i:s",
      filemtime($config[prelude_log])
    )
  ) < 0
  ) {
    // Reset the last access date/time.
    db_prelude_set_last_access(date(
      "Y-m-d H:i:s",
      filemtime($config[prelude_log])
    ));
    // Get the Prelude log file content.
    $content = file($config[prelude_log]);
    $line = 0;
    while ($line < count($content)) {
      // Pattern for beginning of log entry. Date/time in $time_fields.
      if (ereg(
        "^[*]{3} +(Mon|Tue|Wed|Thu|Fri|Sat|Sun) +".
        "(Jan|Feb|Mar|Apr|Jun|Jul|Aug|Sep|Oct|Nov|Dec) +([0-9]+) +".
        "([0-9]{2}:[0-9]{2}:[0-9]{2}) +([0-9]{4})",
        $content[$line],
        $time_fields
      )) {
        // If date/time newer than of last inserted entry into db table,
        // retrieve the other fields from the log entry.
        if (strcmp(
          $last_insert,
          $time_fields[PRELUDE_YEAR]."-".sprintf(
            "%02d",
            month2num($time_fields[PRELUDE_MONTH])
          )."-".sprintf(
            "%02d",
            $time_fields[PRELUDE_DAY]
          )." ".$time_fields[PRELUDE_TIME]
        ) < 0) {
          $plugin = array();
          $received = array();
          $message = array();
          $ip_header = array();
          $tcp_header = array();
          $icmp_header = array();
          $line++;
          // Process the rest of the log entry until EOF or until next log
          // entry found (line starts with three asterisks ["*"]).
          while (
            $line < count($content) &&
            ! ereg("^[*]{3}", $content[$line])
          ) {
            // Now, set some variables using regular expressions. On every
            // line, none or one variable will be set, until at the end of the
            // log entry every variable should be set once.

            // Search pattern for the Plugin line.
            ereg("^Plugin[\t]+: ([^\n]+)\n$", $content[$line], $plugin);
            // Search pattern for the Received line.
            ereg(
              "^received[\t]+: ([^ ]+) times?\n$",
              $content[$line],
              $received
            );
            // Search pattern for the Message line.
            ereg("^message[\t]+: ([^\n]+)\n$", $content[$line], $message);
            // Search pattern for the IP Header line.
            ereg(
              "^Ip hdr +: ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) ->".
              " ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})",
              $content[$line],
              $ip_header
            );
            // Search pattern for the TCP Header line.
            ereg(
              "^Tcp hdr +: ([0-9]+) -> ([0-9]+)",
              $content[$line],
              $tcp_header
            );
            // Search pattern for the ICMP Header line.
            ereg(
              "^Icmp hdr +: type=([0-9]+) code=([0-9]+)\n$",
              $content[$line],
              $icmp_header
            );
            $line++;
          }
          $line--;
          // Test if all regexp variables really ARE all set.
          if (
            $plugin[1] &&
            $received[1] &&
            $message[1] &&
            $ip_header[PRELUDE_SRCIP] &&
            $ip_header[PRELUDE_DESTIP] && (
              // Either the TCP Header variable is set or ICMP Type is not 0.
              (
                $icmp_header[PRELUDE_ICMPTYPE]
              ) ||
              (
                $tcp_header[PRELUDE_TCPSRC] &&
                $tcp_header[PRELUDE_TCPDEST]
              )
            )
          ) {
            // If this IP address is new, put it in $prelude_SRCIPs and in db
            // table for Prelude.
            if (! in_array(
              $ip_header[PRELUDE_SRCIP],
              array_keys($prelude_SRCIPs)
            )) {
              $prelude_SRCIPs[$ip_header[PRELUDE_SRCIP]] = 0;
              db_prelude_new_ip($ip_header[PRELUDE_SRCIP]);
            }
            // If this is a TCP entry, put it into db table.
            if ($tcp_header[PRELUDE_TCPSRC] && $tcp_header[PRELUDE_TCPDEST]) {
              if (! db_prelude_new_log(
                $ip_header[PRELUDE_SRCIP],
                $time_fields[PRELUDE_YEAR]."-".month2num(
                  $time_fields[PRELUDE_MONTH]
                )."-".sprintf("%02d", $time_fields[PRELUDE_DAY])." ".
                $time_fields[PRELUDE_TIME],
                $ip_header[PRELUDE_DESTIP],
                $tcp_header[PRELUDE_TCPSRC],
                $tcp_header[PRELUDE_TCPDEST],
                $plugin[1],
                $received[1],
                $message[1]
              )) {
                die("Insert of log has failed. Quitting.\n");
              }
            // If this is an ICMP entry, put it into db table.
            } elseif ($icmp_header[PRELUDE_ICMPTYPE]) {
              if (! db_prelude_new_log(
                $ip_header[PRELUDE_SRCIP],
                $time_fields[PRELUDE_YEAR]."-".month2num(
                  $time_fields[PRELUDE_MONTH]
                )."-".sprintf("%02d", $time_fields[PRELUDE_DAY])." ".
                $time_fields[PRELUDE_TIME],
                $ip_header[PRELUDE_DESTIP],
                0,
                0,
                $plugin[1],
                $received[1],
                $message[1],
                $icmp_header[PRELUDE_ICMPTYPE],
                $icmp_header[PRELUDE_ICMPCODE]
              )) {
                die("Insert of log has failed. Quitting.\n");
              }
            }
            // Increment the count of this IP in $prelude_SRCIPs and the db
            // table the number of times this entry was received (Prelude can
            // have one log entry for several events).
            for ($cnt = 0; $cnt < $received[1]; $cnt++) {
              $prelude_SRCIPs[$ip_header[PRELUDE_SRCIP]]++;
              if (! db_prelude_inc_count($ip_header[PRELUDE_SRCIP])) {
                die("Increment of counter has failed. Quitting.\n");
              }
            }
            // Update the date/time of the item inserted last into the db table
            // for Prelude.
            db_prelude_set_last_insert(
              $time_fields[PRELUDE_YEAR]."-".
              month2num($time_fields[PRELUDE_MONTH])."-".
              sprintf("%02d", $time_fields[PRELUDE_DAY])." ".
              $time_fields[PRELUDE_TIME]);
          }
        // Else, just skip the whole log entry until next one found.
        } else {
          while (
            $line < count($content) &&
            ! ereg("^[*]{3}", $content[$line])
          ) {
            $line++;
          }
        }
        $line++;
      } else {
        $line++;
      }
    }
  }
}

?>
