<?php

///////////////////////////////////////////////////////////////////////////////
// $Id: index.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
///////////////////////////////////////////////////////////////////////////////
// $Log: index.php,v $
// Revision 1.1.1.1  2002/03/31 00:07:42  richy
// Initial import into CVS (release rel-1_0).
//
///////////////////////////////////////////////////////////////////////////////

/**
* Portscanner/Intrusion Collector
*
* This file collects all loggrep files and displays all info about IP
* addresses. Furthermore, it also reads in the prelude log file, and
* cross-compares the IP addresses. Hence the name Portscan/Intrusion Collector.
*
* It features:
* -caching the info already collected from the loggrep files, only collecting
*  the not already collected loggrep files, so it is a lot faster than it was
*  before I built in the caching feature (it used to take PICker, with about
*  one and a half month of loggrep files, between 50 to 80 seconds on my
*  Pentium 120. Now, with also the prelude feature built in, it takes PICker
*  only a few seconds and info about a certain source IP is usually less than a
*  second, while the old uncached version would start parsing ALL the loggrep
*  files yet again for the zillionth time...). The loggrep files don't change
*  anymore after they have been produced by loggrep, so this has obviously been
*  a very good idea!
* -also collecting the new log entries in the prelude log file (first I tried
*  to use the XML files produced by prelude, but they are invalid and thus, to
*  get it working, would have had to include an XML parser already written by
*  someone else and then changing it to deal with the flaw in the prelude XML
*  files) and cross-referencing the source IP addresses with the source IP
*  addresses from the loggrep files, so I not only see the IP addresses that
*  have been port scanning, but also the IP addresses that have been trying to
*  intrude my system. And of course, also caching this data is a good idea,
*  because an already logged entry won't change anymore. However, the prelude
*  log file IS reparsed every time when the modification time has changed,
*  however already cached entries will be skipped (and a logrotate program
*  should keep this file small through cron, which also means that it is a very
*  good idea to also execute the prelude() function in prelude.inc.php BEFORE
*  the Prelude log file is rotated, or otherwise you will "lose" Prelude
*  events. Just put a tiny php file in /etc/cron.daily with a name BEFORE
*  logrotate, so it is executed just before logrotate and put in it the
*  following:
*    require("/path/to/PICker/include/prelude.inc.php");
*    prelude();
*  and your Prelude events will be cached in the database at least once a day
*  just before logrotate, so everything is well).
* -sending queries about IP addresses (dig, host and whois) to gather info
*  about this IP address. When the whois mode recognizes an abuse email
*  address, it will automatically convert the output to make it a link, that
*  when clicked, produces an automated mail about this IP address, so it is
*  easy to take action.
* -easy sending of an email to the Abuse Team of an ISP, complaining about
*  someone using a certain IP address when portscanning or trying to intrude
*  your system, with the log entries neatly inserted in the email. You can
*  choose between sending the mail in English or in Dutch, with the capability
*  to check or edit any of the fields of the email before sending. This works
*  through the automatic link produced by whois mode and with the
*  "Mail to Abuse" button (where you have to fill in the email address of the
*  Abuse Team yourself, so perhaps it is wise to do an host query first and
*  then trying to send your email to abuse@domain.tld. Works nine out of ten
*  times).
* -if you are suspecting that someone has had different IP addresses while
*  portscanning or trying to intrude your system (either because he/she has a
*  dialup connection or he/she is trying to minimize the chance being caught
*  because the portscans/intrusion attempts are spread over several IP
*  addresses), he/she should normally have IP addresses where only the last one
*  or two octets differ, so you can switch to range mode in PICker, to work
*  with a range of IP addresses (like "211.34.167.20-211.34.167.127" [only last
*  octet differs], "211.34.128.0-211.34.147.255" [last two octets differ], or
*  "0.0.0.0-255.255.255.255", which would give you EVERY IP ADDRESS IN THE
*  WORLD). PICker will search in the Loggrep and Prelude cache db tables and
*  then you can do the same things as with PICker in modes that concern one IP
*  address (performing a dig, host or whois query and sending mail about the
*  IP addresses found in the Loggrep/Prelude caches for this range).
*
* There are several "modes", depending on the setting of some GET/POST
* variables:
* -normal:
*   -condition: "action" or "ip" not set, "sort" may be set
*     -sort: if value is "ip", sort on source IP address ascending, otherwise
*            sort on number of times logged/received descending
*   -result   : parses the Loggrep and Prelude log files for new entries and
*               shows a list of all collected source IP addresses
* -range:
*   -condition: "range" set with begin and end IP of range
*   -result   : retrieves all IP addresses collected in the Loggrep and Prelude
*               db tables for a certain range and displays a list of these
* -info:
*   -condition: "ip" set with source IP address
*   -result   : shows all collected Loggrep/Prelude/Mail/Notes info about this
*               IP address
* -dig:
*   -condition: "action" set to "Dig", "ip" set to source IP address
*   -result   : performs a dig query about source IP address and displays the
*               result
* -host:
*   -condition: "action" set to "Host", "ip" set to source IP address
*   -result   : performs a host query about source IP address and displays the
*               result
* -whois:
*   -condition: "action" set to "Whois", "ip" set to source IP address
*   -result   : performs a whois query about source IP address and displays the
*               result
* -langMail:
*   -condition: "action" set to "langMail", "ip" set to source IP address,
*               "address" set to Abuse Team email address for this source IP
*               address
*   -result   : this will actually be opened in a new window, by means of the
*               JavaScript langMail() function. It will let you choose between
*               producing a Dutch or an English automated email
* -writeMail:
*   -condition: "action" set to "writeMail", "ip" set to source IP address,
*               "address" set to Abuse Team email address for this source IP
*               address, "lang" set to "en" or "nl" for the language to use for
*               the automated email
*   -result   : this will actually be called upon by the JavaScript writeMail()
*               function out of the new window opened in langMail mode.
*               Produces a page where you can check/edit/add any field of the
*               email before sending it to the Abuse Team for this IP address
* -sendMail:
*   -condition: "action" set to "Send mail", "ip" set to source IP address,
*               "from" set to From email address, "to" set to To email address,
*               "subject" set to subject field of the email, "body" set to the
*               body of the email
*   -result   : sends the email to the Abuse Team about this IP address and
*               displays a confirmation page about sending the email
* -langMailR:
*   -condition: "action" set to "langMailR", "range" set to begin and end IP of
*               range, "address" set to Abuse Team email address for these
*               source IP addresses
*   -result   : this will actually be opened in a new window, by means of the
*               JavaScript langMailR() function. It will let you choose between
*               producing a Dutch or an English automated email for the IP
*               addresses found in the range
* -writeMailR:
*   -condition: "action" set to "writeMailR", "range" set to begin and end IP
*               addresses, "address" set to Abuse Team email address for these
*               source IP addresses, "lang" set to "en" or "nl" for the
*               language to use for the automated email
*   -result   : this will actually be called upon by the JavaScript writeMailR()
*               function out of the new window opened in langMailR mode.
*               Produces a page where you can check/edit any field of the
*               email before sending it to the Abuse Team for the IP addresses
*               found in the range
* -sendMailR:
*   -condition: "action" set to "Send mail", "range" set to begin and end IP
*               addresses, "from" set to From email address, "to" set to To
*               email address, "subject" set to subject field of the email,
*               "body" set to the body of the email
*   -result   : sends the email to the Abuse Team about the IP addresses found
*               in the range and displays a confirmation page about sending the
*               email
* -pickIP:
*   -condition: "action" set to "pickIP", "range" set to begin and end IP
*               addresses, "todo" set to the action string that has to be
*               carried out (one of "Dig", "Host" or "Whois")
*   -result   : this will actually be opened in a new window, by means of the
*               JavaScript pickIP() function. It will let you choose between
*               the IP addresses found in the Loggrep/Prelude db tables for a
*               range, so you can perform a dig, host or whois query for this
*               IP address
*
* @module         index
* @modulegroup    main
* @package        PICker
* @version        $Revision: 1.1.1.1 $
* @copyright      (c)2002 R.J. Vrijhof
* @magic          $Id: index.php,v 1.1.1.1 2002/03/31 00:07:42 richy Exp $
* @access         public
*/

set_time_limit(300);

/**
* @include    Initializes the configuration array ($config).
*/
require_once("./include/config.inc.php");

/**
* @include    Contains general functions.
*/
require_once($config[own_dir]."/include/functions.inc.php");

// Record the start time to calculate the number of seconds this script has
// run.
$start = getmicrotime();

// The default sort mode is to sort on the number of logged events for a
// certain IP address in descending order, so you see the topscorers first.
if ($HTTP_GET_VARS["sort"]) {
  $sort_mode = strtolower($HTTP_GET_VARS["sort"]);
} else {
  $sort_mode = "count";
}

/**
* @include    Contains functions specifically for MySQL.
*/
require_once($config[own_dir]."/include/mysql.inc.php");

/**
* @include    Contains functions specifically for Loggrep.
*/
require_once($config[own_dir]."/include/loggrep.inc.php");

/**
* @include    Contains functions specifically for Prelude.
*/
require_once($config[own_dir]."/include/prelude.inc.php");

// Do a quick check for new Loggrep or Prelude logged events. If something new,
// collect in database.
loggrep();
prelude();

/**
* @include    Contains general functions to generate HTML code.
*/
require_once("inc&lib/class.globals.php");
// Instantiate a new Globals object.
$g = new Globals;
// Produce XHTML 1.0.
$g->set("xhtml", true);
// Produce DTD and title (and the rest).
$g->dtd("Portscanner/Intrusion Collector");
// Produce HTML meta tags.
$g->meta();

?>

<style type="text/css">
<!--
a:link, a:visited {
  color : red;
  text-decoration : none
}
a:hover, a:active {
  color : red;
  text-decoration : underline
}
// -->
</style>
<?php

// Normally, how long the script has run will be shown.
$show_time = true;

if (! $HTTP_POST_VARS && ! $HTTP_GET_VARS) {

  /**
  * @include    Contains JavaScript function for Range button.
  * @see        Functions::range_button().
  */
  include_once($config["own_dir"]."/include/range.inc.php");
} elseif ($HTTP_POST_VARS["ip"] || $HTTP_GET_VARS["ip"]) {

  /**
  * @include    Contains JavaScript function for Note button
  * @see        MySQL::db_new_note(), MySQL::db_get_notes().
  */
  include_once($config["own_dir"]."/include/note.inc.php");

  /**
  * @include    Contains JavaScript function for "Mail to Abuse" button.
  * @see        Functions::disp_toolbar().
  */
  include_once($config["own_dir"]."/include/blindMail.inc.php");
} elseif ($HTTP_POST_VARS["range"] || $HTTP_GET_VARS["range"]) {

  /**
  * @include    Contains JavaScript function for "Mail to Abuse" button (for a range).
  * @see        Functions::disp_range_toolbar().
  */
  include_once($config["own_dir"]."/include/blindMailR.inc.php");
}

if ($HTTP_POST_VARS["action"] == "Whois" && $HTTP_POST_VARS["ip"]) {

  /**
  * @include    Contains JavaScript function to invoke langMail mode.
  *
  * Include a tiny JavaScript function that opens a new window where the user
  * can choose between sending the mail in English or in Dutch, but only when
  * in whois mode.
  */
  include_once($config["own_dir"]."/include/langMail.inc.php");
} elseif ($HTTP_GET_VARS["action"] == "pickIP" && $HTTP_GET_VARS["range"] && $HTTP_GET_VARS["todo"]) {
  $show_time = false;

  /**
  * @include    Contains JavaScript function to invoke processIP mode.
  */
  include_once($config["own_dir"]."/include/processIP.inc.php");

} elseif ($HTTP_GET_VARS["action"] == "langMail" && $HTTP_GET_VARS["ip"] && $HTTP_GET_VARS["address"]) {
  $show_time = false;

  /**
  * @include    Contains JavaScript function to invoke writeMail mode.
  *
  * Include a tiny JavaScript function that makes sure either a Dutch or an
  * English mail will be produced when in langMail mode.
  */
  include_once($config["own_dir"]."/include/writeMail.inc.php");

} elseif ($HTTP_GET_VARS["action"] == "langMailR" && $HTTP_GET_VARS["range"] && $HTTP_GET_VARS["address"]) {
  $show_time = false;

  /**
  * @include    Contains JavaScript function to invoke writeMailR mode.
  *
  * Include a tiny JavaScript function that makes sure either a Dutch or an
  * English mail will be produced when in langMailR mode.
  */
  include_once($config["own_dir"]."/include/writeMailR.inc.php");
} elseif ($HTTP_GET_VARS["range"]) {

  /**
  * @include    Contains JavaScript function to invoke pickIP mode.
  *
  * Include a tiny JavaScript function that opens a new window where the user
  * can choose between the IP address found in the range (to do something
  * IP-related, like dig, host or whois).
  */
  include_once($config["own_dir"]."/include/pickIP.inc.php");
}

// Begin of the HTML body of every mode.
echo "\n</head>\n\n<body bgcolor=\"#ffffff\">\n\n";

///////////////////////////////////////////////////////////////////////////////
// dig mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////
if (($HTTP_POST_VARS["action"]  == "Dig" || $HTTP_GET_VARS["action"] == "Dig") && ($HTTP_POST_VARS["ip"] || $HTTP_GET_VARS["ip"])) {
  $ip = ($HTTP_POST_VARS["ip"] ? $HTTP_POST_VARS["ip"] : $HTTP_GET_VARS["ip"]);
  disp_header(1, $ip);
  disp_header(2, "Dig");
  disp_toolbar();
  $cmd = "dig $ip";
  $out = `$cmd`;
  echo nl2br(htmlentities($out))."<br />\n";
  link_list();
  echo "&nbsp;\n";
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// host mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif (($HTTP_POST_VARS["action"]  == "Host" || $HTTP_GET_VARS["action"] == "Host") && ($HTTP_POST_VARS["ip"] || $HTTP_GET_VARS["ip"])) {
  $ip = ($HTTP_POST_VARS["ip"] ? $HTTP_POST_VARS["ip"] : $HTTP_GET_VARS["ip"]);
  disp_header(1, $ip);
  disp_header(2, "Host");
  disp_toolbar();
  $cmd = "host $ip";
  $out = `$cmd`;
  echo nl2br(htmlentities($out))."<br />\n";
  link_list();
  echo "&nbsp;\n";
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// whois mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif (($HTTP_POST_VARS["action"]  == "Whois" || $HTTP_GET_VARS["action"] == "Whois") && ($HTTP_POST_VARS["ip"] || $HTTP_GET_VARS["ip"])) {
  $ip = ($HTTP_POST_VARS["ip"] ? $HTTP_POST_VARS["ip"] : $HTTP_GET_VARS["ip"]);
  disp_header(1, $ip);
  disp_header(2, "Whois");
  disp_toolbar();
  $cmd = "whois $ip 2>&1";
  $temp = `$cmd`;
  $out = nl2br(htmlentities($temp));
  $cnt = 1;
  while (eregi("(abuse@[^] \t\n)}\"']+)", $temp, $regs)) {
    $out = str_replace(
      $regs[1],
      $g->a(
        "javascript: langMail('".rawurlencode($regs[1])."')",
        "Produce automated mail to ".$regs[1]
      ).$regs[1]."</a>",
      $out
    );
    $temp = str_replace($regs[1], "", $temp);
  }
  echo "$out<br />\n";
  link_list();
  echo "&nbsp;\n";
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// langMail mode (see comment on the top of the file about the different modes
// of Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["action"] == "langMail" && $HTTP_GET_VARS["ip"] && $HTTP_GET_VARS["address"]) {

?>
<p>Do you want to send a mail in the English or the Dutch language?</p>

<table border="0" align="center">
  <tr>
    <td>
      <form name="enForm" id="enForm">
        <input type="button" name="enButton" id="enButton" value="English" onclick="writeMail('en')" />
      </form>
    </td>
    <td>
      <form name="nlForm" id="nlForm">
        <input type="button" name="nlButton" id="nlButton" value="Dutch" onclick="writeMail('nl')" />
      </form>
    </td>
  </tr>
</table>
<?php

///////////////////////////////////////////////////////////////////////////////
// writeMail mode (see comment on the top of the file about the different modes
// of Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["action"] == "writeMail" && $HTTP_GET_VARS["ip"] && $HTTP_GET_VARS["address"] && $HTTP_GET_VARS["lang"]) {
  if ($HTTP_GET_VARS["lang"] == "en") {
    disp_header(1, $HTTP_GET_VARS["ip"]);
    disp_header(2, "Send mail to Abuse (English)");
    writeMail();
  } else {
    disp_header(1, $HTTP_GET_VARS["ip"]);
    disp_header(2, "Send mail to Abuse (Dutch)");
    writeMail_nl();
  }
  link_list();
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// sendMail mode (see comment on the top of the file about the different modes
// of Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif (rawurldecode($HTTP_POST_VARS["action"]) == "Send mail" && $HTTP_POST_VARS["ip"] && $HTTP_POST_VARS["from"] && $HTTP_POST_VARS["to"] && $HTTP_POST_VARS["subject"] && $HTTP_POST_VARS["body"]) {
  disp_header(1, $HTTP_POST_VARS["ip"]);
  disp_header(2, "Mail sent to ".rawurldecode($HTTP_POST_VARS["to"]));
  disp_toolbar();

  db_new_mail($HTTP_POST_VARS["ip"], $HTTP_POST_VARS["to"]);
  mail(stripslashes(rawurldecode($HTTP_POST_VARS["to"])), stripslashes(rawurldecode($HTTP_POST_VARS["subject"])), stripslashes(rawurldecode($HTTP_POST_VARS["body"])), "From: ".stripslashes(rawurldecode($HTTP_POST_VARS["from"]))."\nX-Mailer: PICker v".$config[version]."\nbcc: Richard Vrijhof <me@rjvrijhof.com>\n");

  echo "The following mail has been sent:<br /><br />\n";
  echo "<strong>From:</strong> ".htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["from"])))."<br />\n";
  echo "<strong>To:</strong> ".htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["to"])))."<br />\n";
  echo "<strong>Subject:</strong> ".htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["subject"])))."<br />\n";
  echo "<strong>Body:</strong><br />\n".nl2br(htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["body"]))))."<br /><br />\n";

  link_list();
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// langMailR mode (see comment on the top of the file about the different modes
// of Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["action"] == "langMailR" && $HTTP_GET_VARS["range"] && $HTTP_GET_VARS["address"]) {

?>
<p>Do you want to send a mail in the English or the Dutch language?</p>

<table border="0" align="center">
  <tr>
    <td>
      <form>
        <input type="button" value="English" onclick="writeMailR('en')" />
      </form>
    </td>
    <td>
      <form>
        <input type="button" value="Dutch" onclick="writeMailR('nl')" />
      </form>
    </td>
  </tr>
</table>
<?php

///////////////////////////////////////////////////////////////////////////////
// writeMailR mode (see comment on the top of the file about the different
// modes of Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["action"] == "writeMailR" && $HTTP_GET_VARS["range"] && $HTTP_GET_VARS["address"] && $HTTP_GET_VARS["lang"]) {
  if ($HTTP_GET_VARS["lang"] == "en") {
    disp_header(1, $HTTP_GET_VARS["range"]);
    disp_header(2, "Send mail to abuse (English)");
    writeMailR();
  } else {
    disp_header(1, $HTTP_GET_VARS["range"]);
    disp_header(2, "Send mail to abuse (Dutch)");
    writeMailR_nl();
  }
  link_list();
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// sendMailR mode (see comment on the top of the file about the different modes
// of Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif (rawurldecode($HTTP_POST_VARS["action"]) == "Send mail" && $HTTP_POST_VARS["range"] && $HTTP_POST_VARS["from"] && $HTTP_POST_VARS["to"] && $HTTP_POST_VARS["subject"] && $HTTP_POST_VARS["body"]) {
  disp_header(1, $HTTP_POST_VARS["range"]);
  disp_header(2, "Mail sent to ".rawurldecode($HTTP_POST_VARS["to"]));
  disp_range_toolbar();

  db_new_mailR($HTTP_POST_VARS["range"], $HTTP_POST_VARS["to"]);
  mail(stripslashes(rawurldecode($HTTP_POST_VARS["to"])), stripslashes(rawurldecode($HTTP_POST_VARS["subject"])), stripslashes(rawurldecode($HTTP_POST_VARS["body"])), "From: ".stripslashes(rawurldecode($HTTP_POST_VARS["from"]))."\nX-Mailer: PICker v".$config[version]."\nbcc: Richard Vrijhof <me@rjvrijhof.com>\n");

  echo "The following mail has been sent:<br /><br />\n";
  echo "<strong>From:</strong> ".htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["from"])))."<br />\n";
  echo "<strong>To:</strong> ".htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["to"])))."<br />\n";
  echo "<strong>Subject:</strong> ".htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["subject"])))."<br />\n";
  echo "<strong>Body:</strong><br />\n".nl2br(htmlspecialchars(stripslashes(rawurldecode($HTTP_POST_VARS["body"]))))."<br /><br />\n";

  link_list();
  link_ip();

///////////////////////////////////////////////////////////////////////////////
// pickIP mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["action"] == "pickIP" && $HTTP_GET_VARS["range"] && $HTTP_GET_VARS["todo"]) {
  $show_time = false;
  pickIP();

///////////////////////////////////////////////////////////////////////////////
// info mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["ip"]) {
  disp_header(1, $HTTP_GET_VARS["ip"]);
  disp_toolbar();
  if ($HTTP_GET_VARS["action"] == "note" && $HTTP_GET_VARS["note"]) {
    db_new_note($HTTP_GET_VARS["ip"], $HTTP_GET_VARS["note"]);
  }
  // List all collected info about a certain IP address in a nicely formatted
  // table.
  if (db_lg_get_id($HTTP_GET_VARS["ip"])) {
    echo lg_info();
  }
  if (db_prelude_get_id($HTTP_GET_VARS["ip"])) {
    echo prelude_info();
  }
  echo "\n<br />\n";
  mail_info();
  disp_notes();
  link_list();

///////////////////////////////////////////////////////////////////////////////
// range mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////
} elseif ($HTTP_GET_VARS["range"]) {
  disp_header(1, $HTTP_GET_VARS["range"]);
  disp_range_toolbar();
  // List all collected info about a certain IP address in a nicely formatted
  // table.
  $range = explode("-", $HTTP_GET_VARS["range"]);
  $IPs = array_keys(array_merge(lg_range_ips(), prelude_range_ips()));
  $temp = array();
  foreach ($IPs as $val) {
    $temp[] = zero_IP($val);
  }
  sort($temp);
  $IPs = array();
  foreach ($temp as $val) {
    $IPs[] = normalize_IP($val);
  }
  foreach ($IPs as $val) {
    if (db_lg_get_id($val)) {
      echo lg_info($val);
    }
    if (db_prelude_get_id($val)) {
      echo prelude_info($val);
    }
    echo "\n<br />\n";
    mail_info($val);
  }
  link_list();

///////////////////////////////////////////////////////////////////////////////
// normal mode (see comment on the top of the file about the different modes of
// Picker).
///////////////////////////////////////////////////////////////////////////////

} else {
  disp_header(1, "Portscanner/Intrusion Collector");
  range_button();
  sort_link();
  // List all IP addresses collected from the Loggrep files/Prelude log.
  disp_list();
}

///////////////////////////////////////////////////////////////////////////////
// end of mode selection
///////////////////////////////////////////////////////////////////////////////

if ($show_time) {

?>

<p style="font-size : small; font-style : italic">
  This script has taken <strong><?php echo(sprintf("%1.2f", (getmicrotime() - $start)));?></strong>
  seconds to complete.
</p>
<?php

}

if (! ($HTTP_GET_VARS["action"] == "pickIP" && $HTTP_GET_VARS["range"] && $HTTP_GET_VARS["todo"])) {
  copyright();
}

?>

</body>

</html>
